from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist
from django.apps import apps
from django.conf import settings

from eVotUM.Cripto import pkiutils, eccblind
from Crypto.PublicKey import ECC
from evotevotings.admin.models import SecretEntry
import json


def get_election_model():
    return apps.get_model("evoteelections.Election")  # noqa


def get_election_data(election_id=None, user_id=None):
    # Get all necessary data from the database
    try:
        Election = get_election_model()
        election_obj = Election.objects.get(pk=election_id)
        election_cert = election_obj.certificate
        candidates = election_obj.candidates.all().values_list('pk',
                                                               flat=True)
        bulletin = {'election_id': election_id,
                    'candidates': list(candidates)}
        pr_components = election_obj.blindsig_pr_dash_components

        election = {}

        vs_cert = get_certificate(
                    SecretEntry.Identifier.VALIDATOR_CERT
                    )
        votesys_private_key = get_private_key(
                    SecretEntry.Identifier.VOTING_PVT_KEY
                    )
        pvtkey_password = get_private_key_password()

        combo_id = get_user_election(election_id, user_id)

        # Set the blind pr components
        election['pr_dash_components'] = pr_components

        # Set the certificates to be sent
        election['election_cert'] = election_cert
        election['vs_cert'] = vs_cert

        # Set the bulletin information
        election['bulletin'] = json.dumps(bulletin)

        # Sign the needed objects using the system private key
        # Attempt to sign the combo_id {user + election_id}sv
        errorCode, signed_combo_id = pkiutils.signObject(combo_id,
                                                         votesys_private_key,
                                                         pvtkey_password)
        if (errorCode is not None):
            # TODO: Abort and alert (raise exception)
            signed_combo_id = "123456789"

        election['combo_id'] = signed_combo_id

        # Attempt to sign the election_id {election_id}sv
        errorCode, signed_election_id = pkiutils.signObject(
                                            election_id,
                                            votesys_private_key,
                                            pvtkey_password
                                        )
        if (errorCode is not None):
            # TODO: Abort and alert (raise exception)
            signed_election_id = "987654321"

        election['election_id'] = signed_election_id

        return election

    except ObjectDoesNotExist:
        # TODO: Alert for forged election_id
        return False

    return False


def get_certificates(election_id=None):
    certificates = {}

    if(election_id is not None):
        certificates['election'] = get_election_cert(election_id)

    certificates['vs'] = get_certificate(
                            SecretEntry.Identifier.VALIDATOR_CERT
                                )
    pub_key = get_certificate(
                            SecretEntry.Identifier.VALIDATOR_PUB_KEY
                                )
    certificates['vs_pub'] = ECC.import_key(
                                            pub_key,
                                            get_private_key_password()
                                           )
    certificates['root'] = get_certificate(
                                SecretEntry.Identifier.ROOT_CERT
                                )
    certificates['votesys'] = get_certificate(
                                SecretEntry.Identifier.VOTING_CERT
                                )

    return certificates


def get_serialized_bulletin(election_id=None):
    try:
        Election = get_election_model()
        election = Election.objects.get(pk=election_id)
        candidates = election.candidates.all().values_list('pk',
                                                           flat=True)
        return {'election_id': election_id,
                'candidates': list(candidates)}
    except ObjectDoesNotExist:
        # TODO: Alert for forged election_id
        return False

    return False


def get_user_election(election_id=None, user_id=None):
    return {'election_id': str(election_id),
            'user_id': str(user_id)}


def get_election_cert(election_id=None):
    try:
        Election = get_election_model()
        election = Election.objects.get(pk=election_id)
        return election.certificate
    except ObjectDoesNotExist:
        # TODO: Alert for forged election_id
        return False

    return False


def get_certificate(cert_name=None):
    certificate = SecretEntry.objects.filter(
                identifier=cert_name
                )
    if certificate.exists():
        cobj = certificate.first()
        return cobj.body
    else:
        return False


def get_private_key(service_name):
    # TODO: Actual code: Fetch the private from the db
    # TODO: Migrate to general utils app (used by other services)
    return get_certificate(service_name)


def get_private_key_password():
    return settings.SECRET_KEY


def not_expired_certificate(cert=None):
    if(cert is None):
        return False

    # Validate certificate expiration
    (error_code, pubkey) = pkiutils.getPublicKeyFromCertificate(
                                                    cert)

    if (error_code is not None):
        return False

    return True


def validate_certificate(cert=None, root_cert=None):
    """ Checks if a given certificate is chained to a root certificate
        and still before expiration period """
    # Sanity Check
    if(cert is None or root_cert is None):
        return False

    # Check for chain certificates
    chain = pkiutils.verifyCertificateChain(cert,
                                            root_cert)
    if(not chain):
        return -1

    # Check for expired certificate
    expired = not_expired_certificate(root_cert)

    if(not expired):
        return -2

    return True


def initialize_blind_signatures():
    (init_component, pr_component) = eccblind.initSigner()
    return (init_component, pr_component)


def get_extra_election_data(election_id=None, user_id=None):
    # Get all necessary data from the database
    try:
        # Check if Election exists (Exception raised)
        Election = get_election_model()
        Election.objects.get(pk=election_id)

        # Validator Service Certificate
        vs_cert = get_certificate(
                    SecretEntry.Identifier.VALIDATOR_CERT
                    )
        if not vs_cert:
            raise Exception("No Validator Service Certificate found")

        # Voting System Private Key
        votesys_private_key = get_private_key(
                    SecretEntry.Identifier.VOTING_PVT_KEY
                    )
        if not votesys_private_key:
            raise Exception("No Voting System Private Key found")

        # Voting System Private Key password
        pvtkey_password = get_private_key_password()
        if not pvtkey_password:
            raise Exception("No Voting System Pvt Key password found")

        # User/Election combo_id {user + election_id}
        combo_id = get_user_election(election_id, user_id)
        combo_id = json.dumps(combo_id)

        # Sign the needed objects using the system private key
        # Attempt to sign the combo_id {user + election_id}sv
        errorCode, signed_combo_id = pkiutils.signObject(
                                                    combo_id,
                                                    votesys_private_key,
                                                    pvtkey_password)

        if errorCode:
            raise Exception("Could not sign election+user ID")

        # Attempt to sign the election_id {election_id}sv
        errorCode, signed_election_id = pkiutils.signObject(
                                                election_id,
                                                votesys_private_key,
                                                pvtkey_password)

        if errorCode:
            raise Exception("Could not sign election ID")

        return {
            'combo_id': signed_combo_id,
            'vs_cert': vs_cert,
            'election_id': signed_election_id
        }

    except Exception:
        # TODO: Log
        return False
