from django.utils import timezone
from django.db import models
from django.apps import apps

from evoteusers.models import User


class ElectoralProcessQuerySet(models.QuerySet):

    def published(self):
        """Return published Electoral Process (Publication date is greater
        than actual)
        """
        return self.filter(publication_date__lte=timezone.now(),
                           state=self.model.State.OPEN)

    def open(self):
        """Return opened Electoral Process
        """
        return self.filter(state=self.model.State.OPEN)

    def closed(self):
        """Return closed Electoral Process
        """
        return self.filter(state__in=[self.model.State.CLOSED,
                                      self.model.State.ARCHIVED])

    def belongs_to_member(self, user):
        """ """
        processes_ids = user.electoral_members.all().values_list(
            'electoral_process_id', flat=True)
        return self.filter(pk__in=processes_ids)

    def belongs_to_institutional_member(self, user):
        """ """
        return self.filter(created_by=user)

    def belongs_to(self, user):
        """Returns published that belong to user
        """
        ElectionVoter = apps.get_model('evotevoters.ElectionVoter')  # NOQA
        election_ids = ElectionVoter.objects.filter(user=user).values_list(
            'election__electoral_process_id', flat=True)

        return self.published().filter(pk__in=election_ids)

    def public(self):
        """
        Return Electoral Process that are public
        (In state New or Open)
        """
        states = [self.model.State.NEW, self.model.State.OPEN]
        return self.filter(state__in=states)


class ElectoralMemberManager(models.Manager):

    def update_or_create_member(
            self, email, name, designation, role, phone, electoral_process):
        """
        Create new Electoral Member if user exists, otherwise create one
        """
        user, created = User.objects.update_or_create(email=email, defaults={
            'name': name,
            'primary_phone': phone
        })
        member, created = self.update_or_create(
            user=user, electoral_process=electoral_process, defaults={
                'designation': designation,
                'role': role,
                'phone': phone
            })
        return member, created

    def mark_as_responsible(self, electoral_process, user):
        self.filter(electoral_process=electoral_process).update(
            is_comunication_responsible=False)
        user.is_comunication_responsible = True
        user.save()
