from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.views.generic import ListView, DetailView
from django.contrib import messages
from django.conf import settings

from evotevotings.processes.models import ElectoralProcess
from evotevotings.elections.models import Election
from evoteusers.views import EvoteLoginView, PersonalDataBaseView
from evotecore.mixins import DjangoTablesFilterMixin

from evotevotings.elections.utils import get_extra_election_data
from evoteajaxforms.views import FormJsonSimpleView, AjaxResponseAction

from .tables import (ElectoralProcessTable, ElectoralProcessArchiveTable,
                     ElectoralProcessMyTable, ElectionHistoryTable)
from .filters import ArchiveFilter, ElectionHistoryFilter, IndexFilter
from .forms import (ElectionVotingStep1Form, ElectionSearchVoteForm,
                    ContactsForm)
from .utils import get_references
from .mailings import ContactMail


class LoginView(EvoteLoginView):
    """Provides user the ability to login"""
    template_name = "voters/login.html"


class ElectoralProcessListView(DjangoTablesFilterMixin, ListView):
    """
    Shows the main view
    """
    model = ElectoralProcess
    table_pagination = {'per_page': 5}
    filter_class = IndexFilter

    def get_table_class(self):
        if self.request.user.is_authenticated():
            return ElectoralProcessMyTable
        return ElectoralProcessTable

    def get_queryset(self):
        if self.request.user.is_authenticated():
            return ElectoralProcess.objects.belongs_to(self.request.user)
        return ElectoralProcess.objects.published()

    def get_template_names(self):
        if self.request.user.is_authenticated():
            return ["voters/electoralprocess_mylist.html"]
        return ["voters/electoralprocess_list.html"]


class ElectoralProcessDetailView(DetailView):
    """
    Shows the Electoral Process Detail view
    """
    model = ElectoralProcess

    def get_template_names(self):
        if self.request.user.is_authenticated():
            return ["voters/electoralprocess_mydetail.html"]
        return ["voters/electoralprocess_detail.html"]


class ElectoralProcessArchiveView(DjangoTablesFilterMixin, ListView):
    """
    Shows the archive view
    """
    model = ElectoralProcess
    queryset = ElectoralProcess.objects.closed()
    template_name = "voters/electoralprocess_archive.html"

    table_class = ElectoralProcessArchiveTable
    table_pagination = {'per_page': 5}

    filter_class = ArchiveFilter


@method_decorator(login_required, name="dispatch")
class ElectionHistoryView(DjangoTablesFilterMixin, ListView):
    """
    Shows the my elections history view
    """
    model = Election
    template_name = "voters/election_history.html"

    table_class = ElectionHistoryTable
    table_pagination = {'per_page': 5}

    filter_class = ElectionHistoryFilter

    def get_queryset(self, **kwargs):
        """
        Returns the Elections that belong to user order DESC by
        publication_date
        """
        return Election.objects.belongs_to(self.request.user).filter(
            electoral_process__state=ElectoralProcess.State.CLOSED).\
            order_by('-electoral_process__publication_date')


@method_decorator(login_required, name="dispatch")
class ElectoralProcessVotingView(DetailView):
    """
    Shows the voting system
    """
    model = ElectoralProcess
    template_name = "voters/electoralprocess_voting.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep1View(DetailView):
    """
    Shows the voting system
    """
    model = Election
    form_class = ElectionVotingStep1Form
    slug_url_kwarg = "slug_election"
    template_name = "voters/election_voting_step1.html"

    def get_context_data(self, **kwargs):
        context = super(ElectionVotingStep1View, self)\
            .get_context_data(**kwargs)

        form_kwargs = {'instance': self.object}
        self.elect_data = get_extra_election_data(self.object.pk,
                                                  self.request.user.pk)
        if self.elect_data is not False:
            form_kwargs.update(self.elect_data)

        context.update({
            'form': self.form_class(**form_kwargs),
            'is_valid': bool(self.elect_data)
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep2View(DetailView):
    """
    Shows the voting system
    """
    model = Election
    slug_url_kwarg = "slug_election"
    template_name = "voters/election_voting_step2.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep3View(DetailView):
    """
    Shows the voting system
    """
    model = Election
    slug_url_kwarg = "slug_election"
    template_name = "voters/election_voting_step3.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep3PinView(DetailView):
    """
    Shows the voting system
    """
    model = Election
    slug_url_kwarg = "slug_election"
    template_name = "voters/election_voting_step3_pin.html"


@method_decorator(login_required, name="dispatch")
class ElectionVotingStep4View(DetailView):
    """
    Shows the voting system
    """
    model = Election
    slug_url_kwarg = "slug_election"
    template_name = "voters/election_voting_step4.html"


@method_decorator(login_required, name="dispatch")
class ElectionSearchVoteView(DetailView):
    """
    Seach vote in some election
    """
    form_class = ElectionSearchVoteForm
    model = Election
    slug_url_kwarg = "slug_election"
    template_name = "voters/election_search_vote.html"

    def get_context_data(self, **kwargs):
        """ """
        context = super(ElectionSearchVoteView, self).\
            get_context_data(**kwargs)

        context.update({
            'form': self.form_class(self.request.GET),
            'references': get_references()
        })
        return context


class ContactsView(FormJsonSimpleView):
    form_class = ContactsForm
    prefix = 'contacts'
    template_name = "voters/contacts.html"
    action = AjaxResponseAction.REFRESH

    def form_valid(self, form):
        context = {
            'name': form.cleaned_data.get('name', ''),
            'from': form.cleaned_data.get('email', ''),
            'subject': form.cleaned_data.get('subject', ''),
            'message': form.cleaned_data.get('message', '')
        }
        ContactMail(context).send()
        messages.success(
            self.request, _("Your request was submitted successfully."))
        return super(ContactsView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        """ """
        context = super(ContactsView, self).get_context_data(**kwargs)

        context['contact_name'] = getattr(
            settings, "EVOTE_CONTACT_NAME", None)
        context['contact_mail'] = getattr(
            settings, "EVOTE_CONTACT_MAIL", None)
        context['contact_phone'] = getattr(
            settings, "EVOTE_CONTACT_PHONE", None)
        context['contact_address'] = getattr(
            settings, "EVOTE_CONTACT_ADDRESS", None)
        context['contact_map_address'] = getattr(
            settings, "EVOTE_CONTACT_ADDRESS", False)
        context['contact_map_gps'] = getattr(
            settings, "EVOTE_CONTACT_GPS",
            {'lat': 41.5598965, 'lng': -8.3988118})
        return context


@method_decorator(login_required, name="dispatch")
class PersonalDataView(PersonalDataBaseView):
    """ """
    template_name = "voters/personal_data.html"
