#Cripto-7.0.0

import random, os

from Crypto.Util import number
from Crypto.PublicKey import ElGamal
from Crypto import Random
from Crypto.Cipher import AES

from eVotUM.Cripto import pbkdf2

DEFAULT_KEY_SIZE = 1024
ENC_KEY_BLOCK_SIZE = 16
IV_BYTES_LENGTH = 16
AES_KEY_BYTES_LENGTH = 32

#math functions
def getRandom(p):
    k = random.randint(2, p-2)
    while (number.GCD(k, p-1) != 1):
        k += 1
    return k

def blind(publicKey, m):
    k = getRandom(publicKey.p)
    r = pow(publicKey.g, k, publicKey.p)
    h = getRandom(publicKey.p)
    blindM = (h * m) % (publicKey.p - 1)
    return k, r, h, blindM

def sign(privateKey, k, r, blindM):
    return ((blindM - privateKey.x * r) * number.inverse(k, privateKey.p - 1)) % (privateKey.p - 1)

def unblind(publicKey, blindSignature, k, blindM, h):
    return ((number.inverse(h, publicKey.p - 1) - 1) * blindM * number.inverse(k, publicKey.p - 1) + blindSignature) % (publicKey.p - 1)

def verify(publicKey, signature, r, m):
    return (pow(publicKey.g, m, publicKey.p) == (pow(publicKey.y, r, publicKey.p) * pow(r, signature, publicKey.p)) % publicKey.p)

#main functions
#Cripto 7.1.0
def blindData(publicKey, data):
    m = dataToNum(data)
    k, r, h, blindM = blind(publicKey, m)
    return pack([k, r, blindM]), h

#Cripto 7.2.0
def generateBlindSignature(privateKey, components):
    k, r, blindM = unpack(components)
    return sign(privateKey, k, r, blindM)

#Cripto 7.3.0
def unblindSignature(publicKey, blindSignature, components, h):
    k, r, blindM = unpack(components)
    return unblind(publicKey, blindSignature, k, blindM, h)

#Cripto 7.4.0
def verifySignature(publicKey, signature, components, data):
    k, r, blindM = unpack(components)
    m = dataToNum(data)
    return verify(publicKey, signature, r, m)

#key generation
def generateKeyPair(keySize=DEFAULT_KEY_SIZE):
    #1024 ~ 60 seconds
    #2048 ~ 5 minutes
    privateKey = ElGamal.generate(keySize, Random.new().read)
    publicKey = privateKey.publickey()
    return privateKey, publicKey

#files
def importKey(path, passphrase=None):
    f = open(path)
    sKey = f.read()
    f.close()

    if (passphrase):
        sKey = decryptKey(sKey, passphrase)

    try:
        keyComponents = unpack(sKey)
    except:
        return None

    key = ElGamal.ElGamalobj()

    if (len(keyComponents) == 4):
        key.p, key.x, key.y, key.g = keyComponents
    elif (len(keyComponents) == 3):
        key.p, key.y, key.g = keyComponents
    else:
        return None

    return key

def exportKey(key, path, passphrase=None):
    if (key.has_private()):
        sKey = pack([key.p, key.x, key.y, key.g])
    else:
        sKey = pack([key.p, key.y, key.g])

    if (passphrase):
        sKey = encryptKey(sKey, passphrase)

    f = open(path, "w")
    f.write(sKey)
    f.close()

#key encryption
def generateIV():
    return os.urandom(IV_BYTES_LENGTH)

def encryptKey(sKey, passphrase):
    key, salt = pbkdf2.generatePBKDF(passphrase)
    iv = generateIV()
    cipher = AES.new(key[:AES_KEY_BYTES_LENGTH], AES.MODE_CBC, iv)
    return salt.decode("hex") + iv + cipher.encrypt(pkcs5Pad(sKey))

def decryptKey(sKey, passphrase):
    salt = sKey[:pbkdf2.KDF_SALT_BYTES_LENGTH]
    key = pbkdf2.kdf(passphrase, salt.encode("hex"))
    iv = sKey[pbkdf2.KDF_SALT_BYTES_LENGTH:pbkdf2.KDF_SALT_BYTES_LENGTH+IV_BYTES_LENGTH]
    sKey = sKey[pbkdf2.KDF_SALT_BYTES_LENGTH+IV_BYTES_LENGTH:]
    cipher = AES.new(key[:AES_KEY_BYTES_LENGTH], AES.MODE_CBC, iv)
    return pkcs5Unpad(cipher.decrypt(sKey))

def pkcs5Pad(data):
    return data + (ENC_KEY_BLOCK_SIZE - len(data) % ENC_KEY_BLOCK_SIZE) * chr(ENC_KEY_BLOCK_SIZE - len(data) % ENC_KEY_BLOCK_SIZE)

def pkcs5Unpad(data):
    return data[0:-ord(data[-1])]

#utils
def dataToNum(data):
    try:
        return int(data, 16)
    except ValueError:
        return int(data.encode("hex"), 16)

def pack(l):
    return ".".join(map(str, l))

def unpack(s):
    return map(int, s.split("."))
